#!/usr/bin/env python

# Authors:Xiaohong Zhuang, Dr.Jeffery B. Klauda
#    Date:9/27/2016

# Calculate the number of clusters and number of lipids in each cluster
import sys
import ast
import numpy as np
from sklearn.cluster import DBSCAN

#################################################################################################################################

# Update Cutoff distance in Anstromg

Dcut = 5.5;


###################################################################################################################################

Nframe = 1;

# Read name of file
fob = open('ftype.txt','r')
ftype = fob.readline()
ftype = ftype.strip()
fob.close()

# Read current number of dyn
fob = open('Idyn.txt','r')
Idyn = fob.readline()
Idyn = Idyn.strip()
Idyn = int(Idyn)
fob.close()

# Read current number of frame
fob = open('Frame.txt','r')
Fr = fob.readline()
Fr = Fr.strip()
Fr = int(Fr)
fob.close()

# Read name of file
filetoread1 = "%s/box-fix-%i.txt" % (ftype,Idyn)
data1=np.genfromtxt(filetoread1,delimiter="\t")
left=data1[:,0]; right=data1[:,1]; box=data1[:,2];

# Read the data file
filetoread2 = "%s/xy-fix-%i-%i.dat" % (ftype,Idyn,Fr)
Data = np.genfromtxt(filetoread2,delimiter="\t")
data = Data[:,3:]

# Find number of rows and columns
NR= np.shape(data)[0]; NC= np.shape(data)[1];

filetowrite2 = "%s/num-%i-%i.txt" % (ftype,Idyn,Fr)
f2 = open(filetowrite2,'w')
if Fr ==0:
	# Write title, index of frame, Index of cluster, number of lipids in each cluster
	f2.write('%s\t%s\t%s\t%s\n' % ('Idyn','If2','Ic','Nlc'))

filetowrite3 = "%s/tot-%i-%i.txt" % (ftype,Idyn,Fr)
f3 = open(filetowrite3,'w')
if Fr ==0:
	# Write title, index of frame, number of lipid per frame, number of cluster per frame, number of lipids per cluster [frame averaged]
	f3.write('%s\t%s\t%s\t%s\t%s\n' % ('Idyn','If1','Nlf','Ncf','Nlca'))  

# Find line number
def LNR(x_pt,y_pt):
	with open(filetoread2) as myFile:
    		for num, line in enumerate(myFile, 1):
        		if format(x_pt,'.4f') in line and format(y_pt,'.4f') in line:
				cols = line.strip().split();
				resn=cols[0]; resid=cols[1]; nim=cols[2];  # retrun the resn,resid and image index
            			return resn, resid, nim

Nline = 0

#################################################################################################################################################

# print title for the first frame
if Fr==0:
	print 'line','\t','Idyn''\t','If','\t','Ic','\t','Ilip','\t','Resn','\t','Resid','\t','Nimg','\t','X_coord','\t','Y_coord'

def run_dbscan(X,fr):
	line = Nline;
	# Compute DBSCAN
	db = DBSCAN(eps=Dcut, min_samples=3).fit(X)
	#print db.get_params(deep=True)
	labels = db.labels_
	components = db.components_
	# Number of clusters in labels, ignoring noise if present.
	n_clusters_ = len(set(labels)) - (1 if -1 in labels else 0)
	#print 'C0','tot',n_clusters_ ;        # number of total clusters	
	clusters_1=[];clusters_2=[];clusters_3=[];

	# Find size of each cluster
	if n_clusters_ > 0:
		# Find clusters have coordinate inside main image
		clusters_1 = [];
		for i in range(0,n_clusters_,1):
			comp_1 = X[labels==i]
			check1=any(xy[0]>left[fr] and xy[0]<right[fr] and xy[1]>left[fr] and xy[1]<right[fr] for xy in comp_1)
			if check1 == True:
				clusters_1.append(comp_1) 

	# Find clusters not dulipicate due to the boudary ,i.e. not share any residue, use the last cluster
	if len(clusters_1) > 0:	
		clusters_2=[];
		Lresid_ac = [];
		# Check if resid dulplicate inside each cluster		
		for i in range(0,len(clusters_1),1):
			comp_1c=clusters_1[i];
			# Find resid of all lipids in the clusters
			Lresid_1c=[];
			for p in range(0,len(comp_1c),1):
				Resn, Resid, Nim = LNR(comp_1c[p][0],comp_1c[p][1]);
				Lresid_1c.append(Resid);
			for q in range(0,len(comp_1c),1):
				Occ=Lresid_1c.count(Lresid_1c[q])
				if Occ > 1:
					for j in range (0,len(comp_1c),1):
						# Find line in the coordinate file
						Resn, Resid, Nim = LNR(comp_1c[j][0],comp_1c[j][1]);
						# print line, frame, cluster, component and x y coordinate in the cluster
						print line,'\t',Idyn-1,'\t',fr,'\t',i,'\t',j,'\t',Resn,'\t',Resid,'\t',Nim,'\t',comp_1c[j][0],'\t',comp_1c[j][1]
						line+=1
					print "Error! Please use lower Dcut!"
					print "Resid",Lresid_1c[i],"is duplicated",Occ, "times in a cluster for Idyn(dyn index)=",Idyn-1,",If(frame index)=",fr,",Ic(cluster index)=",i
					sys.exit()
			Lresid_ac.append(Lresid_1c);

		# Check if resid dulplicate inside among clusters	
		for i in range(0,len(clusters_1),1):
			comp_2=clusters_1[i];
			check2list=[];
			for k in range(i+1,len(clusters_1),1):
				check2=any(r in Lresid_ac[i] for r in Lresid_ac[k])				
				check2list.append(check2)
			if True not in check2list:
				clusters_2.append(comp_2)

		# save frame number and tot lipids in cluster, and number of clusters, and frame averaged cluster size
		Sc=sum(len(x) for x in clusters_2); Nc=len(clusters_2);
		f3.write('%i\t%i\t%i\t%i\t%2.4f\n' % (Idyn-1,fr,Sc,Nc,Sc/float(Nc)))
	
	if len(clusters_2) > 0:
		# Print the number of lipid in each clusters of the last frame,i.e., cluster size, and maybe components when needed
		if fr >= 0 :    
			for i in range(0,len(clusters_2),1):
				comp_f=clusters_2[i];
				# save frame number and number of clusters
				f2.write('%i\t%i\t%i\t%i\n' % (Idyn-1,fr,i,len(comp_f)))
				for j in range (0,len(comp_f),1):
					# Find line in the coordinate file
					Resn, Resid, Nim = LNR(comp_f[j][0],comp_f[j][1]);
					# print line, frame, cluster, component and x y coordinate in the cluster
					print line,'\t',Idyn-1,'\t',fr,'\t',i,'\t',j,'\t',Resn,'\t',Resid,'\t',Nim,'\t',comp_f[j][0],'\t',comp_f[j][1]
					line+=1

#######################################################################################################################################################################

# calculte clustering of single block
run_dbscan(data,Fr);

